// This class shows the UI for the Actuators app

package lab.tutorial.restclient;

import java.util.ArrayList;

import lab.tutorial.restclient.actuators.Actuator;
import lab.tutorial.restclient.actuators.ActuatorConstants;
import lab.tutorial.restclient.actuators.OnOffSwitch;

import org.apache.http.client.ClientProtocolException;
import org.apache.http.conn.ConnectTimeoutException;
import org.json.JSONException;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemClickListener;

public class ActuatorsActivity extends Activity {
	
	// Local data
	private ArrayList<Actuator> actuatori;
	// UI elements
	private ListView lv;
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        // UI initialisation
        setContentView(R.layout.actuators);
        lv = (ListView) findViewById(R.id.listView);
        
        // Server data initialisation
        final String protocol = "http://";
        final String host = "embedded.cs.pub.ro/";
        final String serverURL = "si/smarthome/";
        final String format = ".json";
        
        // Read data from server
        actuatori = new ArrayList<Actuator>();
        String URI = protocol + host + serverURL + ActuatorConstants.ACTUATOR_TYPE + ActuatorConstants.ONOFF + format;
        try {
        	actuatori = JSONParser.getActuators(URI,ActuatorConstants.ONOFF);
        } catch (JSONException e) {
            e.printStackTrace();
            Toast.makeText(getApplicationContext(), "Response from server is not a proper JSON", Toast.LENGTH_SHORT).show();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        	Toast.makeText(getApplicationContext(), "Not an HTTP server", Toast.LENGTH_SHORT);
        } catch (ConnectTimeoutException e) {
            e.printStackTrace();
            Toast.makeText(getApplicationContext(), "Server unresponsive", Toast.LENGTH_SHORT).show();
        }

        // Configure UI for the sensor list
        final ListWithImageAdapter la = new ListWithImageAdapter(this,R.layout.list_item, actuatori);
        lv.setAdapter(la);
        // What happens when you click a button in the Sensor list
            lv.setOnItemClickListener(new OnItemClickListener() {
	    	    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
	    	    	try{
		    	    	OnOffSwitch o = (OnOffSwitch) actuatori.get(position);
		    	    	String newValue = o.onValue;
		    	    	if (o.getSetting()) newValue = o.offValue;
		    	    	String settingURI = protocol + host + serverURL + 
		    	    		ActuatorConstants.ACTUATOR + "/" + o.getId() + ActuatorConstants.SETTING + newValue + format;
		    	    	o.setSetting(JSONParser.confirmSetting(settingURI));
		    	    	la.notifyDataSetChanged();
	    	        } catch (JSONException e) {
	    	            e.printStackTrace();
	    	            Toast.makeText(getApplicationContext(), "Response from server is not a proper JSON", Toast.LENGTH_SHORT).show();
	    	        } catch (ClientProtocolException e) {
	    	            e.printStackTrace();
	    	        	Toast.makeText(getApplicationContext(), "Not an HTTP server", Toast.LENGTH_SHORT);
	    	        } catch (ConnectTimeoutException e) {
	    	            e.printStackTrace();
	    	            Toast.makeText(getApplicationContext(), "Server unresponsive", Toast.LENGTH_SHORT).show();
	    	        }
	    	    }
	        });
     }
    
    // What happens when you press the back button on your phone
    @Override
    public void onBackPressed() {
    }
    
    // What happens when you press the menu button on your phone
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu, menu);
        return true;
    }
    // What happens when you press a button in the Options menu
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        switch (item.getItemId()) {
        case R.id.sensors:
        	Intent sIntent = new Intent(this, SensorsActivity.class);
        	startActivity(sIntent);
        	return true;
        case R.id.actuators:
        	Intent aIntent = new Intent(this, ActuatorsActivity.class);
        	startActivity(aIntent);
            return true;
        default:
            return super.onOptionsItemSelected(item);
        }
    }
    
    // Custom adapter to populate the sensors list
    private class ListWithImageAdapter extends ArrayAdapter<Actuator> {

    	private ArrayList<Actuator> items;

    	public ListWithImageAdapter(Context context, int textViewResourceId, ArrayList<Actuator> items) {
                super(context, textViewResourceId, items);
                this.items = items;
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
                View v = convertView;
                if (v == null) {
                    LayoutInflater vi = (LayoutInflater)getSystemService(Context.LAYOUT_INFLATER_SERVICE);
                    v = vi.inflate(R.layout.list_item, null);
                }
                
                Actuator a = items.get(position);
                OnOffSwitch o = (OnOffSwitch)a;
                if (o != null) {
                        TextView t = (TextView) v.findViewById(R.id.txtName);
                        if (t != null) {
                              t.setText(o.getName());                            
                        }
                        t = (TextView) v.findViewById(R.id.txtLocation);
                        if (t != null) {
                              t.setText(o.getLocation());                            
                        }
                        t = (TextView) v.findViewById(R.id.txtValue);
                        if (t != null) {
                        	  if (o.getSetting()){
                        		  t.setText(o.onValue);
                        	  } else {
                        		  t.setText(o.offValue);
                        	  }
                        }
                        ImageView i = (ImageView) v.findViewById(R.id.img);
                        if (i != null) {
                        	if (o.getSetting()){
                        		i.setImageResource(R.drawable.light_switch_on);
                        	} else {
                        		i.setImageResource(R.drawable.light_switch_off);
                        	}
                        }
                }
                return v;
        }
    }
}