// This class handles JSON parsing

package lab.tutorial.restclient;

import java.sql.Timestamp;
import java.util.ArrayList;

import lab.tutorial.restclient.actuators.Actuator;
import lab.tutorial.restclient.actuators.ActuatorConstants;
import lab.tutorial.restclient.actuators.OnOffSwitch;
import lab.tutorial.restclient.sensors.Sensor;
import lab.tutorial.restclient.sensors.SensorConstants;
import lab.tutorial.restclient.sensors.TemperatureSensor;
import lab.tutorial.restclient.sensors.ValuesHistory;
import lab.tutorial.restclient.sensors.ValuesHistory.Pair;

import org.apache.http.client.ClientProtocolException;
import org.apache.http.conn.ConnectTimeoutException;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

public class JSONParser {
	// extract a device ID from a JSON description
	public static int extractId(JSONObject json) throws JSONException {
		return json.getInt("id");
	}

	// extract a device name from a JSON description
	public static String extractName(JSONObject json) throws JSONException {
		return json.getString("name");
	}

	// extract a device location from a JSON description
	public static String extractLocation(JSONObject json) throws JSONException {
		return json.getString("location");
	}
	
	// extract a device type from a JSON description
	public static String extractType(JSONObject json) throws JSONException {
		return json.getString("type");
	}
	
	// extract an actuator setting from a JSON description
	public static String extractSetting(JSONObject json) throws JSONException {
		return json.getString("setting");
	}
	
	// change an actuator setting and confirm
	public static String confirmSetting(String URI) throws JSONException, ClientProtocolException, ConnectTimeoutException {
		JSONObject response = RestComm.restPut(URI);
		if (response != null) {
			return response.getJSONObject("actuator").getString("setting");
		}
		return null;
	}
	
	// get list of sensors
	public static ArrayList<Sensor> getSensors(String URI, String type) throws JSONException, ClientProtocolException, ConnectTimeoutException {
	
		ArrayList<Sensor> senzori = new ArrayList<Sensor>();
		JSONObject response = RestComm.restGet(URI);
		if (response != null) {
			JSONArray sList = response.getJSONArray(SensorConstants.SENSOR_SET);
			for (int i = 0; i < sList.length();i++) {
				// Daca este un senzor de temperatura il adaugam in lista ca atare
				if (JSONParser.extractType(sList.getJSONObject(i)).equalsIgnoreCase(type)){
					senzori.add(new TemperatureSensor(sList.getJSONObject(i)));
				}
			}
		} 
		return senzori;
	}
	
	// get list of actuators
	public static ArrayList<Actuator> getActuators(String URI, String type) throws JSONException, ClientProtocolException, ConnectTimeoutException {
		
		ArrayList<Actuator> actuatori = new ArrayList<Actuator>();
		JSONObject response = RestComm.restGet(URI);
		if (response != null) {
			JSONArray aList = response.getJSONArray(ActuatorConstants.ACTUATOR_SET);
			for (int i = 0; i < aList.length();i++) {
				// Daca este un senzor de temperatura il adaugam in lista ca atare
				if (JSONParser.extractType(aList.getJSONObject(i)).equalsIgnoreCase(type)){
					actuatori.add(new OnOffSwitch(aList.getJSONObject(i)));
				}
			}
		}
		return actuatori;
	}
	
	// get list of values for a sensor
	public static ArrayList<Pair<Timestamp,Double>> getSensorValues(String URI) throws JSONException, ClientProtocolException, ConnectTimeoutException {
		
		ArrayList<Pair<Timestamp,Double>> values = new ArrayList<Pair<Timestamp,Double>>();
		JSONObject response = RestComm.restGet(URI);
		if (response != null) {
			JSONArray vList = response.getJSONArray(SensorConstants.VALUE_SET);
			for (int i = 0; i < vList.length();i++) {
				values.add(toValue(vList.getJSONObject(i)));
			}
		}
		return values;
	}

	// convert a JSON value object to an actual value pair (timestamp and value)
	public static ValuesHistory.Pair<Timestamp,Double> toValue(JSONObject j){
		try {
			return new ValuesHistory.Pair<Timestamp,Double>(new Timestamp(j.getLong("timestamp")),
					j.getDouble("valoare"));			
		} catch (JSONException e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
}
