// This class shows the UI for the Sensors app

package lab.tutorial.restclient;

import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.PriorityQueue;

import lab.tutorial.restclient.sensors.Sensor;
import lab.tutorial.restclient.sensors.SensorConstants;
import lab.tutorial.restclient.sensors.TemperatureSensor;
import lab.tutorial.restclient.sensors.ValuesHistory;

import org.apache.http.client.ClientProtocolException;
import org.apache.http.conn.ConnectTimeoutException;
import org.json.JSONException;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.DashPathEffect;
import android.graphics.LinearGradient;
import android.graphics.Paint;
import android.graphics.Shader;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ViewFlipper;
import android.widget.AdapterView.OnItemClickListener;

import com.androidplot.Plot;
import com.androidplot.series.XYSeries;
import com.androidplot.xy.BoundaryMode;
import com.androidplot.xy.LineAndPointFormatter;
import com.androidplot.xy.SimpleXYSeries;
import com.androidplot.xy.XYPlot;
import com.androidplot.xy.XYStepMode;

public class SensorsActivity extends Activity {
	
	// Local data
	private ArrayList<Sensor> senzori;
	// UI elements
	private ListView lv;
	private XYPlot mySimpleXYPlot;
	private ViewFlipper flipper;
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        // UI initialisation
        setContentView(R.layout.sensors);
        flipper = (ViewFlipper) findViewById(R.id.flipper);
        lv = (ListView) findViewById(R.id.listView);
        mySimpleXYPlot = (XYPlot) findViewById(R.id.mySimpleXYPlot);
        
        // Server data initialisation
        final String protocol = "http://";
        final String host = "embedded.cs.pub.ro/";
        final String serverURL = "si/smarthome/";
        final String format = ".json";
        
        // Read data from server
        senzori = new ArrayList<Sensor>();
        String URI = protocol + host + serverURL + SensorConstants.SENSOR_TYPE + SensorConstants.TEMPERATURE + format;
        try {
        	senzori = JSONParser.getSensors(URI, SensorConstants.TEMPERATURE);
        	for(int i=0; i<senzori.size(); i++){
				String valURI = protocol + host + serverURL + SensorConstants.VALUE_OF_SENSOR + senzori.get(i).getId() + format;
				senzori.get(i).addToHistory(JSONParser.getSensorValues(valURI));
        	}
        } catch (JSONException e) {
            e.printStackTrace();
            Toast.makeText(getApplicationContext(), "Response from server is not a proper JSON", Toast.LENGTH_SHORT).show();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        	Toast.makeText(getApplicationContext(), "Not an HTTP server", Toast.LENGTH_SHORT);
        } catch (ConnectTimeoutException e) {
        	e.printStackTrace();
            Toast.makeText(getApplicationContext(), "Server unresponsive", Toast.LENGTH_SHORT).show();
        }
        
        // Configure UI for the sensor list
        final ListWithImageAdapter la = new ListWithImageAdapter(this,R.layout.list_item, senzori);
        lv.setAdapter(la);
        // What happens when you click a button in the Sensor list
        lv.setOnItemClickListener(new OnItemClickListener() {
    	    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
    	    	flipper.showNext();
    	    	createChart(senzori.get(position));
    	    }
        });
    }
    
    @SuppressWarnings("unchecked") // thery are ValuesHistory.Pair<Timestamp,Double>, trust me
	private void createChart(Sensor s){
    	mySimpleXYPlot.clear();

    	// get data for chart - hackish because "The Iterator provided in method iterator() 
    	// is not guaranteed to traverse the elements of the PriorityQueue in any particular order."
    	PriorityQueue<ValuesHistory.Pair<Timestamp,Double>> v = s.getHistory();
    	
    	Object[] v2 = v.toArray();
    	Arrays.sort(v2);
    	
    	int count = s.getHistorySize();
    	Number[] values = new Number[count];
    	Number[] timestamps = new Number[count];
    	
    	for (int i=0;i<count;i++) {
    		// Difference between SQL Timestamp and Java Timestamp, got to *1000
    		timestamps[i] = ((ValuesHistory.Pair<Timestamp,Double>)v2[i]).getLeft().getTime()*1000;
    		values[i] = ((ValuesHistory.Pair<Timestamp,Double>)v2[i]).getRight();
    	}
  
    	// create our series from our array of nums:
        XYSeries series = new SimpleXYSeries(
                Arrays.asList(timestamps),
                Arrays.asList(values),
                "Temperatura de la senzor");
 
        mySimpleXYPlot.getGraphWidget().getGridBackgroundPaint().setColor(Color.WHITE);
        mySimpleXYPlot.getGraphWidget().getGridLinePaint().setColor(Color.BLACK);
        mySimpleXYPlot.getGraphWidget().getGridLinePaint().setPathEffect(new DashPathEffect(new float[]{1,1}, 1));
        mySimpleXYPlot.getGraphWidget().getDomainOriginLinePaint().setColor(Color.BLACK);
        mySimpleXYPlot.getGraphWidget().getRangeOriginLinePaint().setColor(Color.BLACK);
        mySimpleXYPlot.getGraphWidget().getDomainLabelPaint().setTextSize(16);
        mySimpleXYPlot.getGraphWidget().getRangeLabelPaint().setTextSize(16);
        mySimpleXYPlot.getGraphWidget().getCursorLabelPaint().setTextSize(16);
 
        mySimpleXYPlot.setBorderStyle(Plot.BorderStyle.SQUARE, null, null);
        mySimpleXYPlot.getBorderPaint().setStrokeWidth(1);
        mySimpleXYPlot.getBorderPaint().setAntiAlias(false);
        mySimpleXYPlot.getBorderPaint().setColor(Color.WHITE);
 
        // setup our line fill paint to be a slightly transparent gradient:
        Paint lineFill = new Paint();
        lineFill.setAlpha(200);
        lineFill.setShader(new LinearGradient(0, 0, 0, 250, Color.WHITE, Color.GREEN, Shader.TileMode.MIRROR));
 
        LineAndPointFormatter formatter  = new LineAndPointFormatter(Color.rgb(0, 0,0), Color.BLUE, Color.RED);
        formatter.setFillPaint(lineFill);
        mySimpleXYPlot.getGraphWidget().setPadding(2, 2, 15, 10);
        mySimpleXYPlot.addSeries(series, formatter);
        mySimpleXYPlot.setRangeBoundaries(-5, 40, BoundaryMode.FIXED);
        
        // draw a domain tick for each entry:
        mySimpleXYPlot.setDomainStep(XYStepMode.SUBDIVIDE, timestamps.length);
 
        // customize our domain/range labels
        mySimpleXYPlot.setDomainLabel("Ora");
        if (s.getType() == SensorConstants.TEMPERATURE) {
        	TemperatureSensor st = (TemperatureSensor)s;
        	mySimpleXYPlot.setRangeLabel(st.getName()+" ("+st.getUnit()+")");
        }
 
        // get rid of decimal points in our range labels:
       // mySimpleXYPlot.setRangeValueFormat(new DecimalFormat("0"));
 
        mySimpleXYPlot.setDomainValueFormat(new SimpleDateFormat("k:mm"));
 
        // by default, AndroidPlot displays developer guides to aid in laying out your plot.
        // To get rid of them call disableAllMarkup():
        mySimpleXYPlot.disableAllMarkup();
    }
    
    // What happens when you press the back button on your phone
    @Override
    public void onBackPressed() {
    	flipper.showPrevious();
    }
    
    // What happens when you press the menu button on your phone
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu, menu);
        return true;
    }
    // What happens when you press a button in the Options menu
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        switch (item.getItemId()) {
        case R.id.sensors:
        	Intent sIntent = new Intent(this, SensorsActivity.class);
        	startActivity(sIntent);
        	return true;
        case R.id.actuators:
        	Intent aIntent = new Intent(this, ActuatorsActivity.class);
        	startActivity(aIntent);
            return true;
        default:
            return super.onOptionsItemSelected(item);
        }
    }
    
    // Custom adapter to populate the sensors list
    private class ListWithImageAdapter extends ArrayAdapter<Sensor> {

    	private ArrayList<Sensor> items;

    	public ListWithImageAdapter(Context context, int textViewResourceId, ArrayList<Sensor> items) {
                super(context, textViewResourceId, items);
                this.items = items;
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
                View v = convertView;
                if (v == null) {
                    LayoutInflater vi = (LayoutInflater)getSystemService(Context.LAYOUT_INFLATER_SERVICE);
                    v = vi.inflate(R.layout.list_item, null);
                }
                
                Sensor s = items.get(position);
                if (s != null) {
                        TextView t = (TextView) v.findViewById(R.id.txtName);
                        if (t != null) {
                              t.setText(s.getName());                            
                        }
                        t = (TextView) v.findViewById(R.id.txtLocation);
                        if (t != null) {
                              t.setText(s.getLocation());                            
                        }
                        t = (TextView) v.findViewById(R.id.txtValue);
                        if (t != null) {
                              t.setText(((TemperatureSensor)s).getValueAndUnit());                            
                        }
                        ImageView i = (ImageView) v.findViewById(R.id.img);
                        if (i != null) {
                        	if (s.getType()==SensorConstants.TEMPERATURE) {
                        		i.setImageResource(R.drawable.temperature);
                        	}
                        }
                }
                return v;
        }
    }
}