// This class provides a way of storing values registered by a sensor
// as a priority queue of pairs (timestamp, value) sorted by timestamp

package lab.tutorial.restclient.sensors;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.PriorityQueue;

public class ValuesHistory {

	private PriorityQueue<Pair<Timestamp,Double>> values;
	private int capacity;
	
	public ValuesHistory() {
		capacity = 10;
		this.values = new PriorityQueue<Pair<Timestamp,Double>>(capacity);
	}

	public int getCapacity() {
		return capacity;
	}

	public PriorityQueue<Pair<Timestamp,Double>> getValues() {
		return values;
	}

	public void setValues(PriorityQueue<Pair<Timestamp,Double>> values) {
		this.values = values;
	}

	public void addValues(ArrayList<Pair<Timestamp,Double>> newValues) {
		for (int i = 0; i < newValues.size();i++)
			values.add(newValues.get(i));
	}

	public static class Pair<L,R> implements Comparable<Pair<L,R>> {

		  private final L left;
		  private final R right;
		
		  public Pair(L left, R right) {
		    this.left = left;
		    this.right = right;
		  }
		
		  public L getLeft() { return left; }
		  public R getRight() { return right; }

    	  @Override
	  	  public int compareTo(Pair<L,R> arg0) {
    		  // store latest first
    		  return (-1)*((Timestamp)this.left).compareTo((Timestamp)arg0.left); 
		  }

		}

}
